#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "scanapi.h"

typedef struct _scanner_data {

	unsigned int port;

} SCANNER_DATA;

char callback(void *userData, unsigned long *ips, unsigned int elements);

int main(int argc, char **argv)
{
	extern char *optarg;
	SCANNER_DATA *data = (SCANNER_DATA *)malloc(sizeof(SCANNER_DATA));
	SCANAPI_CTX  ctx;
	int c = 0;

	scanapiDefaults(&ctx);

	if (argc == 1)
	{
		fprintf(stdout,"Usage: %s [-t subnet[/bitmask|netmask]] [-n threads] [-i ips per thread] [-p port] [-v]\n",
					argv[0]);
		return 0;
	}

	while ((c = getopt(argc, argv, "t:n:i:p:v")) != EOF)
	{
		switch (c)
		{
			case 't':
				ctx.target   = optarg;
				break;		
			case 'n':
				ctx.threads  = atoi(optarg);
				break;
			case 'i':
				ctx.ipPerThread = atoi(optarg);
				break;
			case 'p':
				data->port = atoi(optarg);
				break;
			case 'v':
				ctx.verbose = 1;
				break;
		}
	}

	if ((!ctx.threads) || (!ctx.target) || (!ctx.ipPerThread) || (!data->port))
	{
		fprintf(stdout,"One or more arguments specified was invalid.\n");
		return 0;
	}

	if (!scanapiInitialize(&ctx, data, callback))
		return 0;

	scanapiStart(&ctx);

	scanapiDeinitialize(&ctx);

	free(data);

	return 1;
}

char callback(void *userData, unsigned long *ips, unsigned int elements)
{
	unsigned int x, numLeft = elements, ret;
	struct {

		struct sockaddr_in s;
	
		time_t connectTime;
		int    fd;

		char   isDone;

	} connectInfo[elements];

	memset(connectInfo, 0, sizeof(connectInfo));

	for (x = 0; x < elements; x++)
	{
		int set;
		connectInfo[x].fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

		if (!connectInfo[x].fd)
			return 0;

		connectInfo[x].connectTime = time(NULL);
		connectInfo[x].isDone      = 0;
	
		fcntl(connectInfo[x].fd, F_GETFL, &set);		
		set |= O_NONBLOCK;
		fcntl(connectInfo[x].fd, F_SETFL, set);

		connectInfo[x].s.sin_family = AF_INET;
		connectInfo[x].s.sin_port   = htons(((SCANNER_DATA *)userData)->port);
		connectInfo[x].s.sin_addr.s_addr = ips[x];
	}	

	while (numLeft > 0)
	{
		for (x = 0; x < elements; x++)
		{
			if (connectInfo[x].isDone)
				continue;

			connect(connectInfo[x].fd, (struct sockaddr *)&connectInfo[x].s, sizeof(struct sockaddr));

			ret = errno;

			if (ret == EISCONN)
			{
				fprintf(stdout,"%hi.%hi.%hi.%hi has port %d open.\n",
							((unsigned char *)&ips[x])[0],
							((unsigned char *)&ips[x])[1],
							((unsigned char *)&ips[x])[2],
							((unsigned char *)&ips[x])[3],
							((SCANNER_DATA *)userData)->port);
				connectInfo[x].isDone = 1;
				numLeft--;
			}
			else if ((ret == EALREADY) || (ret == EINPROGRESS))
			{
				if (time(NULL) - connectInfo[x].connectTime >= 5)
				{
					connectInfo[x].isDone = 1;
					numLeft--;
				}
			}
			else
			{
				connectInfo[x].isDone = 1;
				numLeft--;
			}
		}

		usleep(200);
	}

	for (x = 0; x < elements; x++)
		close(connectInfo[x].fd);

	return 1;
}
