/*
 * uninformed  -- IP scanning API using IPBlocks
 *
 * skape
 * 06/13/2002
 */
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <time.h>

#include "scanapi.h"

typedef struct _scanapi_thread_args {

	SCANAPI_CTX   *ctx;

	unsigned long *ips;
	unsigned int  elements;

} SCANAPI_THREAD_ARGS;

void _scanapiBaseThread(SCANAPI_THREAD_ARGS *args);

void scanapiDefaults(SCANAPI_CTX *ctx)
{
	ctx->target      = NULL;
	ctx->userData    = NULL;
	ctx->threads     = 10;
	ctx->ipPerThread = 32;
	ctx->verbose     = 0;
}

short scanapiInitialize(SCANAPI_CTX *ctx, void *userData, char (*callback)(void *userData, unsigned long *ips, unsigned int ipCount))
{
	ctx->userData       = userData;
	ctx->callback       = callback;
	ctx->currentThreads = 0;

	if (ctx->target)
		ctx->ipblock = ipblocksCreateIpBlock(ctx->target, 128, 0);

	pthread_mutex_init(&ctx->currentThreadsMutex, NULL);

	return 1;
}

short scanapiStart(SCANAPI_CTX *ctx)
{
	time_t startSeconds = time(NULL);

	if ((!ctx->ipblock) || (!ctx->callback))
		return 0;

	if (ctx->verbose)
		fprintf(stdout, "SCANAPI: Starting scan of %lu IP's at %d IP's per %d threads...\n", ipblocksGetIpCount(ctx->ipblock), ctx->ipPerThread, ctx->threads);

	while (1)
	{
		SCANAPI_THREAD_ARGS *args;

		args = (SCANAPI_THREAD_ARGS *)malloc(sizeof(SCANAPI_THREAD_ARGS));
	
		if (!args)
			return 0;

		args->ips = (unsigned long *)malloc(sizeof(unsigned long) * ctx->ipPerThread);

		if (!args->ips)
		{
			free(args);
			return 0;
		}

		args->ctx = ctx;

		if ((args->elements = ipblocksEnumerate(ctx->ipblock, args->ips, ctx->ipPerThread)))
		{
			pthread_t id;

			scanapiThreadCountIncrement(ctx);

			pthread_create(&id, NULL, (void *(*)(void *))_scanapiBaseThread, (void *)args);

			while (scanapiThreadCountGet(ctx) >= ctx->threads)
				usleep(1000);
		}
		else
		{
			free(args->ips);
			free(args);
			break;
		}
	}

	while (scanapiThreadCountGet(ctx) > 0)
		usleep(1000);

	if (ctx->verbose)
		fprintf(stdout,"Scan complete.  Operation took %lu seconds.\n",time(NULL) - startSeconds);

	return 1;
}

short scanapiDeinitialize(SCANAPI_CTX *ctx)
{
	if (ctx->ipblock)
		ipblocksDestroyIpBlock(ctx->ipblock);

	pthread_mutex_destroy(&ctx->currentThreadsMutex);

	return 1;
}

void _scanapiBaseThread(SCANAPI_THREAD_ARGS *args)
{
	pthread_detach(pthread_self());

	args->ctx->callback(args->ctx->userData, args->ips, args->elements);

	scanapiThreadCountDecrement(args->ctx);

	free(args->ips);
	free(args);
}

void scanapiThreadCountIncrement(SCANAPI_CTX *ctx)
{
	pthread_mutex_lock(&ctx->currentThreadsMutex);
	ctx->currentThreads++;
	pthread_mutex_unlock(&ctx->currentThreadsMutex);
}

void scanapiThreadCountDecrement(SCANAPI_CTX *ctx)
{
	pthread_mutex_lock(&ctx->currentThreadsMutex);
	ctx->currentThreads--;
	pthread_mutex_unlock(&ctx->currentThreadsMutex);
}

short scanapiThreadCountGet(SCANAPI_CTX *ctx)
{
	short threadCount = 0;

	pthread_mutex_lock(&ctx->currentThreadsMutex);
	threadCount = ctx->currentThreads;
	pthread_mutex_unlock(&ctx->currentThreadsMutex);

	return threadCount;
}
